// MultiServer Demo

#include <SPI.h>
#include <Ethernet.h>

byte      mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 0, 100);

EthernetServer server1(23);
EthernetServer server2(23);

void setup()
{
  Serial.begin(9600);
  while (!Serial);

  // start the Ethernet connection and the server:
  Ethernet.begin(mac, ip);
  server1.begin();
  server2.begin();
  Serial.print("\nServer is listening at ");
  Serial.println(Ethernet.localIP());
}

void loop()
{
  EthernetClient client2;
  
  // listen for incoming clients
  EthernetClient client1 = server1.available();
  if (client1) {
    Serial.println("Client A connected.");
    client1.println("Greetings, program!");
    
    while(client1.available()>0) client1.read(); // Gobble

    // Loop while client is connected.
    while (client1.connected())
    {
        client2 = server2.available();
        if (client2.connected()) {
          Serial.println("Client B connected. Getting rid of them...");
          client2.println("\nThe system is busy. Try back later.\n");
          delay(1);
          client2.stop();
          Serial.println("Client B disconnected.");
        }
      // Then handle the actual client.      
      
      // If data is available, just read it and write it back to the user.
      if (client1.available())
      {
        char c = client1.read();
        if (c>0) Serial.write(c);
      }
    } // end of while... go back and do it again.
    
    // If here, we must no longer be connected.
    delay(1);
    // close the connection:
    client1.stop();
    Serial.println("Client A disconnected.");
  }
}

/*--------------------------------------------------------------------------*/
// 2015-02-15: Verified against Arduino 1.6.0.
//
// To fix the Ethernet library so it correctly allows multiple connections
// to the same port, the following files will need to be modified:
//
// libraries/Ethernet/src/Ethernet.h
// libraries/Ethernet/src/Ethernet.cpp
//
// libraries/Ethernet/src/EthernetClient.cpp
// libraries/Ethernet/src/EthernetClient.h
//
// libraries/Ethernet/src/EthernetServer.cpp
//
// On Mac OS X, these are embedded inside the Arduino.app package. Browse
// to that in the Finder, then right-click and select "Show Package
// Contents" and then you can go to:
//
// Contents/Resources/Java/libraries
//
// Is there a better way?
/*

Modify the following files:

1) Ethernet.h: The Ethernet object currently only tracks which Port the
socket is listening to. Add the following array to hold the remote Port. 

Add this after static "uint16_t _server_port[MAX_SOCK_NUM];"

  // ACH - added
  static uint16_t _client_port[MAX_SOCK_NUM]; // ACH

2) Ethernet.cpp: Add the declaraction of the new array.

Add this after "uint16_t EthernetClass::_server_port[MAX_SOCK_NUM]"

// ACH - added
uint16_t EthernetClass::_client_port[MAX_SOCK_NUM] = { 0, 0, 0, 0 }; // ACH
 
3) EthernetClient.h: Add prototypes for the new functions, and declare a
new local variable that will track the destination port of this client.

Add this in the private: section
 
  // ACH - added
  uint16_t _dstport; // ACH

  // ACH - added
  uint8_t *getRemoteIP(uint8_t remoteIP[]); // ACH
  uint16_t getRemotePort(); // ACH

4)  EthernetClient.cpp: When the Client object is stopped, it resets the
_server_port to zero. We should probably do this for the new _client_port.
In void EthernetClient::stop():

Add after this: EthernetClass::_server_port[_sock] = 0;

  // ACH - added
  EthernetClass::_client_port[_sock] = 0; // ACH

Add these two functions at the bottom of the file:

// ACH - added
uint8_t *EthernetClient::getRemoteIP(uint8_t remoteIP[]) // ACH
{
  W5100.readSnDIPR(_sock, remoteIP);
  return remoteIP;
}
 
uint16_t EthernetClient::getRemotePort() // ACH
{
  return W5100.readSnDPORT(_sock);
}
 
5) EthernetServer.cpp: This code has to be modified so when it checks for
a connection, it checks both the Port (existing code) AND the remote
client's port (new code). If the connection has never been made, it will
initialize the remote port varaible correctly.

Add the following code to available()

EthernetClient EthernetServer::available()
{
  accept();
 
  for (int sock = 0; sock < MAX_SOCK_NUM; sock++) {
    EthernetClient client(sock);
    if (EthernetClass::_server_port[sock] == _port &&
        (client.status() == SnSR::ESTABLISHED ||
         client.status() == SnSR::CLOSE_WAIT)) {
 
      // ACH - added
      // See if we have identified this one before
      if (EthernetClass::_client_port[sock] == 0 ) {
          client._dstport = client.getRemotePort();
          EthernetClass::_client_port[sock] = client._dstport;
          return client;
      }
      if (EthernetClass::_client_port[sock] != client._dstport) {
        // Not us!
        continue;
      }
      // ACH - end of additions
      //if (client.available()) { // ACH - comment out
        // XXX: don't always pick the lowest numbered socket.
        return client;
      //} // ACH - comment out
    }
  }
 
  return EthernetClient(MAX_SOCK_NUM);
}
 
...and code to write():

size_t EthernetServer::write(const uint8_t *buffer, size_t size)
{
  size_t n = 0;
 
  accept();
 
  for (int sock = 0; sock < MAX_SOCK_NUM; sock++) {
    EthernetClient client(sock);
 
    if (EthernetClass::_server_port[sock] == _port &&
      // ACH - added
      EthernetClass::_client_port[sock] == client._srcport && // ACH
      client.status() == SnSR::ESTABLISHED) {
      n += client.write(buffer, size);
    }
  }
 
  return n;
}

*/
/*--------------------------------------------------------------------------*/

